@@ -20,6 +20,7 @@
 #include "gconf-property-editor.h"
 #include "file-transfer-dialog.h"
 #include "gnome-theme-installer.h"
+#include "theme-thumbnail.h"
 
 #define GTK_THEME_KEY      "/desktop/gnome/interface/gtk_theme"
 #define WINDOW_THEME_KEY   "/desktop/gnome/applications/window_manager/theme"
@@ -36,10 +37,20 @@
 
 static void read_themes (GladeXML *dialog);
 
+enum
+{
+  META_THEME_NAME_COLUMN,
+  META_THEME_ID_COLUMN,
+  META_PIXBUF_COLUMN,
+  META_DEFAULT_THEME_COLUMN,
+  META_xN_COLUMNS
+};
+
 enum
 {
   THEME_NAME_COLUMN,
   THEME_ID_COLUMN,
+  PIXBUF_COLUMN,
   DEFAULT_THEME_COLUMN,
   N_COLUMNS
 };
@@ -72,6 +83,113 @@ create_dialog (void)
 }
 
 
+static void
+load_meta_themes (GtkTreeView *tree_view,
+		  GList       *meta_theme_list,
+		  char        *current_theme,
+		  char        *default_theme)
+{
+  GList *list;
+  GtkTreeModel *model;
+  GtkWidget *swindow;
+  gint i = 0;
+  gboolean current_theme_found = FALSE;
+  GtkTreeRowReference *row_ref = NULL;
+
+  swindow = GTK_WIDGET (tree_view)->parent;
+  model = gtk_tree_view_get_model (tree_view);
+
+  setting_model = TRUE;
+  gtk_list_store_clear (GTK_LIST_STORE (model));
+  
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
+				  GTK_POLICY_NEVER, GTK_POLICY_NEVER);
+  gtk_widget_set_usize (swindow, -1, -1);
+
+  for (list = meta_theme_list; list; list = list->next)
+    {
+      GnomeThemeMetaInfo *meta_theme_info = list->data;
+      gchar *blurb;
+      GtkTreeIter iter;
+      gboolean is_default;
+      GdkPixbuf *pixbuf;
+      gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
+
+      if (strcmp (default_theme, meta_theme_info->name) == 0)
+	is_default = TRUE;
+      else
+	is_default = FALSE;
+
+      if (strcmp (current_theme, meta_theme_info->name) == 0)
+	{
+	  GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
+	  row_ref = gtk_tree_row_reference_new (model, path);
+	  gtk_tree_path_free (path);
+	  current_theme_found = TRUE;
+	}
+      blurb = g_strdup_printf ("<span size=\"larger\" weight=\"bold\">%s</span>\n\n%s",
+			       meta_theme_info->readable_name, meta_theme_info->comment);
+      pixbuf = generate_theme_thumbnail (meta_theme_info);
+      gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+			  PIXBUF_COLUMN, pixbuf,
+			  THEME_NAME_COLUMN, blurb,
+			  THEME_ID_COLUMN, meta_theme_info->name,
+			  DEFAULT_THEME_COLUMN, is_default,
+			  -1);
+      g_free (blurb);
+
+      if (i == MAX_ELEMENTS_BEFORE_SCROLLING)
+	{
+	  GtkRequisition rectangle;
+	  gtk_widget_size_request (GTK_WIDGET (tree_view), &rectangle);
+	  gtk_widget_set_usize (swindow, -1, rectangle.height);
+	  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
+					  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+	}
+      i++;
+    }
+
+  if (! current_theme_found)
+    {
+      GtkTreeIter iter;
+      GtkTreePath *path;
+      gboolean is_default;
+
+      if (strcmp (default_theme, current_theme) == 0)
+	is_default = TRUE;
+      else
+	is_default = FALSE;
+      gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
+      gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+			  THEME_NAME_COLUMN, current_theme,
+			  THEME_ID_COLUMN, current_theme,
+			  DEFAULT_THEME_COLUMN, is_default,
+			  -1);
+
+      path = gtk_tree_model_get_path (model, &iter);
+      row_ref = gtk_tree_row_reference_new (model, path);
+      gtk_tree_path_free (path);
+    }
+
+  if (row_ref)
+    {
+      GtkTreePath *path;
+
+      path = gtk_tree_row_reference_get_path (row_ref);
+      gtk_tree_view_set_cursor (tree_view,path, NULL, FALSE);
+
+      if (initial_scroll)
+	{
+	  gtk_tree_view_scroll_to_cell (tree_view, path, NULL, TRUE, 0.5, 0.0);
+	  initial_scroll = FALSE;
+	}
+      
+      gtk_tree_path_free (path);
+      gtk_tree_row_reference_free (row_ref);
+    }
+  setting_model = FALSE;
+}
+
 static void
 load_theme_names (GtkTreeView *tree_view,
 		  GList       *theme_list,
@@ -117,6 +235,7 @@ load_theme_names (GtkTreeView *tree_view,
 	}
       gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 			  THEME_NAME_COLUMN, name,
+			  THEME_ID_COLUMN, name,
 			  DEFAULT_THEME_COLUMN, is_default,
 			  -1);
 
@@ -144,6 +263,7 @@ load_theme_names (GtkTreeView *tree_view,
       gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
       gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 			  THEME_NAME_COLUMN, current_theme,
+			  THEME_ID_COLUMN, current_theme,
 			  DEFAULT_THEME_COLUMN, is_default,
 			  -1);
 
@@ -189,7 +309,7 @@ update_gconf_key_from_selection (GtkTreeSelection *selection,
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
       gtk_tree_model_get (model, &iter,
-			  THEME_NAME_COLUMN, &new_key,
+			  THEME_ID_COLUMN, &new_key,
 			  -1);
     }
   else
@@ -221,73 +341,34 @@ static void
 meta_theme_setup_info (GnomeThemeMetaInfo *meta_theme_info,
 		       GladeXML           *dialog)
 {
+  GtkWidget *notebook;
+  GtkWidget *toggle;
+  gulong signal_id;
+
+  notebook = WID ("meta_theme_notebook");
+
+  /* Undo all signals */
+  toggle = WID ("meta_theme_font1_toggle");
+
   if (meta_theme_info == NULL)
     {
-      gtk_widget_hide (WID ("meta_theme_extras_vbox"));
-      gtk_widget_hide (WID ("meta_theme_description_label"));
-      gtk_image_set_from_pixbuf (GTK_IMAGE (WID ("meta_theme_image")), NULL);
+      gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
     }
   else
     {
-      if (meta_theme_info->icon_file)
+      if (meta_theme_info->application_font != NULL)
 	{
-	  gtk_image_set_from_file (GTK_IMAGE (WID ("meta_theme_image")), meta_theme_info->icon_file);
-	}
-      else
-	{
-	  gtk_image_set_from_pixbuf (GTK_IMAGE (WID ("meta_theme_image")), NULL);
-	}
-      if (meta_theme_info->comment)
-	{
-	  gchar *real_comment;
-
-	  real_comment = g_strconcat ("<span size=\"larger\" weight=\"bold\">",
-				      meta_theme_info->comment,
-				      "</span>", NULL);
-	  gtk_label_set_markup (GTK_LABEL (WID ("meta_theme_description_label")),
-				real_comment);
-	  g_free (real_comment);
-	  gtk_widget_show (WID ("meta_theme_description_label"));
-	}
-      else
-	{
-	  gtk_widget_hide (WID ("meta_theme_description_label"));
-	}
-
-      if (meta_theme_info->font != NULL)
-	{
-	  gtk_widget_show (WID ("meta_theme_extras_vbox"));
-	  if (meta_theme_info->background != NULL)
-	    {
-	      gtk_label_set_text (GTK_LABEL (WID ("meta_theme_info_label")),
-				  _("This theme suggests the use of a font and a background:"));
-	      gtk_widget_show (WID ("meta_theme_background_button"));
-	      gtk_widget_show (WID ("meta_theme_font_button"));
-	    }
+	  if (meta_theme_info->background_image != NULL)
+	    gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 3);
 	  else
-	    {
-	      gtk_label_set_text (GTK_LABEL (WID ("meta_theme_info_label")),
-				  _("This theme suggests the use of a font:"));
-	      gtk_widget_hide (WID ("meta_theme_background_button"));
-	      gtk_widget_show (WID ("meta_theme_font_button"));
-	    }
+	    gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 1);
 	}
       else
 	{
-	  if (meta_theme_info->background != NULL)
-	    {
-	      gtk_widget_show (WID ("meta_theme_extras_vbox"));
-	      gtk_label_set_text (GTK_LABEL (WID ("meta_theme_info_label")),
-				  _("This theme suggests the use of a background:"));
-	      gtk_widget_show (WID ("meta_theme_background_button"));
-	      gtk_widget_hide (WID ("meta_theme_font_button"));
-	    }
+	  if (meta_theme_info->background_image != NULL)
+	    gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 2);
 	  else
-	    {
-	      gtk_widget_hide (WID ("meta_theme_extras_vbox"));
-	      gtk_widget_hide (WID ("meta_theme_background_button"));
-	      gtk_widget_hide (WID ("meta_theme_font_button"));
-	    }
+	    gtk_notebook_set_current_page (GTK_NOTEBOOK (notebook), 0);
 	}
     }
 }
@@ -339,7 +420,7 @@ meta_theme_selection_changed (GtkTreeSelection *selection,
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
       gtk_tree_model_get (model, &iter,
-			  THEME_NAME_COLUMN, &meta_theme_name,
+			  THEME_ID_COLUMN, &meta_theme_name,
 			  -1);
     }
   else
@@ -379,7 +460,7 @@ window_theme_selection_changed (GtkTreeSelection *selection,
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
       gtk_tree_model_get (model, &iter,
-			  THEME_NAME_COLUMN, &window_theme_name,
+			  THEME_ID_COLUMN, &window_theme_name,
 			  -1);
     }
   else
@@ -419,6 +500,12 @@ read_themes (GladeXML *dialog)
   gchar *current_icon_theme;
   GnomeWindowManager *window_manager;
   GnomeWMSettings wm_settings;
+  GtkWidget *notebook;
+
+  gboolean have_meta_theme;
+  gboolean have_gtk_theme;
+  gboolean have_window_theme;
+  gboolean have_icon_theme;
 
   client = gconf_client_get_default ();
 
@@ -435,6 +522,8 @@ read_themes (GladeXML *dialog)
   if (current_gtk_theme == NULL)
     current_gtk_theme = g_strdup ("Default");
 
+  notebook = WID ("theme_notebook");
+  
   /* First, we update the GTK+ themes page */
   theme_list = gnome_theme_info_find_by_type (GNOME_THEME_GTK_2);
   string_list = NULL;
@@ -444,14 +533,34 @@ read_themes (GladeXML *dialog)
       string_list = g_list_prepend (string_list, info->name);
     }
 
-  load_theme_names (GTK_TREE_VIEW (WID ("control_theme_treeview")), string_list, current_gtk_theme, GTK_THEME_DEFAULT_NAME);
-  g_list_free (string_list);
+  if (string_list == NULL)
+    {
+      gtk_widget_hide (WID ("control_theme_vbox"));
+      have_gtk_theme = FALSE;
+    }
+  else
+    {
+      gtk_widget_show (WID ("control_theme_vbox"));
+      have_gtk_theme = TRUE;
+      load_theme_names (GTK_TREE_VIEW (WID ("control_theme_treeview")), string_list, current_gtk_theme, GTK_THEME_DEFAULT_NAME);
+      g_list_free (string_list);
+    }
   g_list_free (theme_list);
 
   /* Next, we do the window managers */
   string_list = gnome_window_manager_get_theme_list (window_manager);
-  load_theme_names (GTK_TREE_VIEW (WID ("window_theme_treeview")), string_list, current_window_theme, WINDOW_THEME_DEFAULT_NAME);
-  g_list_free (string_list);
+  if (string_list == NULL)
+    {
+      gtk_widget_hide (WID ("window_theme_vbox"));
+      have_window_theme = FALSE;
+    }
+  else
+    {
+      gtk_widget_show (WID ("window_theme_vbox"));
+      have_window_theme = TRUE;
+      load_theme_names (GTK_TREE_VIEW (WID ("window_theme_treeview")), string_list, current_window_theme, WINDOW_THEME_DEFAULT_NAME);
+      g_list_free (string_list);
+    }
 
   /* Third, we do the icon theme */
   theme_list = gnome_theme_icon_info_find_all ();
@@ -463,8 +572,18 @@ read_themes (GladeXML *dialog)
       string_list = g_list_prepend (string_list, info->name);
     }
 
-  load_theme_names (GTK_TREE_VIEW (WID ("icon_theme_treeview")), string_list, current_icon_theme, ICON_THEME_DEFAULT_NAME);
-  g_list_free (string_list);
+  if (string_list == NULL)
+    {
+      gtk_widget_hide (WID ("icon_theme_vbox"));
+      have_icon_theme = FALSE;
+    }
+  else
+    {
+      gtk_widget_show (WID ("icon_theme_vbox"));
+      have_icon_theme = TRUE;
+      load_theme_names (GTK_TREE_VIEW (WID ("icon_theme_treeview")), string_list, current_icon_theme, ICON_THEME_DEFAULT_NAME);
+      g_list_free (string_list);
+    }
   g_list_free (theme_list);
 
   /* Finally, we do the Meta themes */
@@ -480,23 +599,47 @@ read_themes (GladeXML *dialog)
 	{
 	  current_meta_theme = g_strdup (info->name);
 	}
-      string_list = g_list_prepend (string_list, info->name);
+      string_list = g_list_prepend (string_list, info);
     }
 
-  if (current_meta_theme == NULL)
-    current_meta_theme = g_strdup (_("Current modified"));
-
-  load_theme_names (GTK_TREE_VIEW (WID ("meta_theme_treeview")), string_list, current_meta_theme, META_THEME_DEFAULT_NAME);
-  g_list_free (string_list);
+  if (string_list == NULL)
+    {
+      have_meta_theme = FALSE;
+      gtk_widget_hide (WID ("meta_theme_hbox"));
+    }
+  else
+    {
+      have_meta_theme = TRUE;
+      gtk_widget_show (WID ("meta_theme_hbox"));
+      if (current_meta_theme == NULL)
+	current_meta_theme = g_strdup (_("Current modified"));
+      load_meta_themes (GTK_TREE_VIEW (WID ("meta_theme_treeview")), string_list, current_meta_theme, META_THEME_DEFAULT_NAME);
+      g_list_free (string_list);
+    }
   g_list_free (theme_list);
   
   
   g_free (current_gtk_theme);
   g_free (current_icon_theme);
   g_free (current_meta_theme);
+  
+  if (! have_meta_theme && ! have_icon_theme && !have_window_theme && ! have_gtk_theme)
+    {
+      GtkWidget *dialog;
+
+      dialog = gtk_message_dialog_new (NULL,
+				       GTK_DIALOG_MODAL,
+				       GTK_MESSAGE_ERROR,
+				       GTK_BUTTONS_OK,
+				       _("No themes could be found on your system.  This probably means that your \"Theme Preferences\" dialog was improperly installed."));
+      gtk_dialog_run (GTK_DIALOG (dialog));
+      gtk_widget_destroy (dialog);
+      exit (0);
+    }
 }
 
 
+
 static void
 theme_key_changed (GConfClient *client,
 		   guint        cnxn_id,
@@ -694,14 +837,27 @@ setup_tree_view (GtkTreeView *tree_view,
 {
   GtkTreeModel *model;
   GtkTreeSelection *selection;
- 
+  GtkCellRenderer *renderer;
+
+  renderer = g_object_new (GTK_TYPE_CELL_RENDERER_PIXBUF,
+			   "xpad", 4,
+			   "ypad", 4,
+			   NULL);
+			   
+
   gtk_tree_view_insert_column_with_attributes (tree_view,
  					       -1, NULL,
- 					       gtk_cell_renderer_text_new (),
- 					       "text", THEME_NAME_COLUMN,
+ 					       renderer,
+ 					       "pixbuf", PIXBUF_COLUMN,
  					       NULL);
-  
-  model = (GtkTreeModel *) gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_tree_view_insert_column_with_attributes (tree_view,
+ 					       -1, NULL,
+ 					       renderer,
+ 					       "markup", THEME_NAME_COLUMN,
+ 					       NULL);
+
+  model = (GtkTreeModel *) gtk_list_store_new (N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_BOOLEAN);
   gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model), 0, sort_func, NULL, NULL);
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), 0, GTK_SORT_ASCENDING);
   gtk_tree_view_set_model (tree_view, model);
@@ -710,17 +866,23 @@ setup_tree_view (GtkTreeView *tree_view,
   g_signal_connect (G_OBJECT (selection), "changed", changed_callback, dialog);
 }
 
+
 static void
 setup_dialog (GladeXML *dialog)
 {
   GConfClient *client;
   GtkWidget *parent, *widget;
   GnomeWindowManager *window_manager;
-
+  GtkSizeGroup *size_group;
+  
   client = gconf_client_get_default ();
   window_manager = gnome_wm_manager_get_current (gdk_display_get_default_screen (gdk_display_get_default ()));
   parent = WID ("theme_dialog");
 
+  size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
+  gtk_size_group_add_widget (size_group, WID ("meta_theme_install_button"));
+  gtk_size_group_add_widget (size_group, WID ("meta_theme_save_button"));
+  
   setup_tree_view (GTK_TREE_VIEW (WID ("meta_theme_treeview")),
   		   (GCallback) meta_theme_selection_changed,
 		   dialog);
@@ -774,7 +936,9 @@ setup_dialog (GladeXML *dialog)
   g_signal_connect (G_OBJECT (widget), "clicked",
 		    G_CALLBACK (show_manage_themes), dialog);
 
-  /*
+
+
+/*
   g_signal_connect (G_OBJECT (WID ("install_dialog")), "response",
 		    G_CALLBACK (install_dialog_response), dialog);
   */
@@ -807,6 +971,9 @@ main (int argc, char *argv[])
   bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
+
+  setup_theme_thumbnail_factory (argc, argv);
+  
   gnome_program_init ("gnome-theme-properties", VERSION,
 		      LIBGNOMEUI_MODULE, argc, argv,
 		      GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
